package com.gct.gateway.filter;

import com.gct.gateway.util.TokenUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author: NewMeanning
 * @create: 2021-01-11 16:49
 **/
@Slf4j
public class TokenFilter implements GlobalFilter, Ordered {

    //过滤器排除的url列表
    private static final String[] excludedAuthPages = {
            "/login",   //登陆
            "/logout", //登出
            "/swagger-ui.html",  //swagger
            "/swagger-resources",
            "/v2/api-docs",
            "/checkCode.jpg", //验证码
            "/actuator",
            "/authA11Token"
    };

    //自定义过滤器实现
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        String pathValue = request.getPath().value();
        log.info("requestPath:{}",pathValue);
        //判断path是否为排除url
        boolean pathFlag = false;
        for(int i=0;i<excludedAuthPages.length;i++){
            if(pathValue.indexOf(excludedAuthPages[i])!=-1){
                pathFlag = true; break;
            }
        }
        if(!pathFlag){
            List<String> tokens = request.getHeaders().get("token");
            if(tokens!=null&&tokens.size()>0){ //token 是否为空
                String token = tokens.get(0);
                Boolean tokenExpired = TokenUtils.isTokenExpired(token);
                log.info("tokenExpired:{}",tokenExpired);
                if(tokenExpired){ //token 是否过期
                    return errorReturn(exchange,"-999","登陆过期");
                }
            }else {
                //token 为空返回
                return errorReturn(exchange,"-998","未登录");
            }
        }
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 100;
    }


    private Mono<Void> errorReturn(ServerWebExchange exchange,String status,String message){
        return Mono.defer(() -> {
            exchange.getResponse().setStatusCode(HttpStatus.OK);//设置status
            final ServerHttpResponse response = exchange.getResponse();
            response.getHeaders().set("Content-Type","text/html;charset=UTF-8");
            String jsonString = "{\"status\":\""+status+"\",\"message\":\""+message+"\"}";
            byte[] bytes = jsonString.getBytes(StandardCharsets.UTF_8);
            DataBuffer buffer = exchange.getResponse().bufferFactory().wrap(bytes);
            log.info("TokenFilter拦截非法请求，没有检测到token or token 过期............");
            return response.writeWith(Flux.just(buffer));//设置body
        });
    }
}
